package io.tswf.minecraft.blockventure.core.transmutation

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.AtExit
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.ClassReference
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.MethodDescription
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.Self
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.Transmute
import io.tswf.groovy.bettergroovy.asm.transmutation.meta.MissingTargetStrategy
import io.tswf.minecraft.blockventure.core.view.ViewDynamicRegistryManagerImmutable
import net.minecraft.registry.DynamicRegistryManager

import java.util.stream.Stream

@Slf4j
@CompileStatic
class TransmutationDynamicRegistryManagerImmutable {

    static void injectIntoDynamicRegistries(Map registriesMap) {
        log.info("Injecting into ${registriesMap}!")
    }

    @Transmute(
            targetClass = @ClassReference(DynamicRegistryManager.ImmutableImpl),
            targetMethod = @MethodDescription(
                    name = "<init>",
                    returnType = @ClassReference(descriptor = "V"),
                    parameterTypeClasses = [ List ]
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void injectIntoDynamicRegistries(List registriesMap, @Self Object self) {
        def view = (ViewDynamicRegistryManagerImmutable) self
        def mutableRegistries = new HashMap<>(view.registries)
        view.registries = mutableRegistries

        injectIntoDynamicRegistries(mutableRegistries)
    }

    @Transmute(
            targetClass = @ClassReference(DynamicRegistryManager.ImmutableImpl),
            targetMethod = @MethodDescription(
                    name = "<init>",
                    returnType = @ClassReference(descriptor = "V"),
                    parameterTypeClasses = [ Map ]
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void injectIntoDynamicRegistries(Map registriesMap, @Self Object self) {
        def view = (ViewDynamicRegistryManagerImmutable) self
        def mutableRegistries = new HashMap<>(view.registries)
        view.registries = mutableRegistries

        injectIntoDynamicRegistries(mutableRegistries)
    }

    @Transmute(
            targetClass = @ClassReference(DynamicRegistryManager.ImmutableImpl),
            targetMethod = @MethodDescription(
                    name = "<init>",
                    returnType = @ClassReference(descriptor = "V"),
                    parameterTypeClasses = [ Stream ]
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void injectIntoDynamicRegistries(Stream registriesMap, @Self Object self) {
        def view = (ViewDynamicRegistryManagerImmutable) self
        def mutableRegistries = new HashMap<>(view.registries)
        view.registries = mutableRegistries

        injectIntoDynamicRegistries(mutableRegistries)
    }

}
